"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const cors_1 = __importDefault(require("cors"));
const moment_1 = __importDefault(require("moment"));
const log_1 = __importDefault(require("./log"));
const routes_1 = __importDefault(require("./routes"));
const path_to_regexp_1 = require("path-to-regexp");
const path_1 = __importDefault(require("path"));
class Server {
    constructor(port, host, data) {
        this.port = port;
        this.host = host;
        this.app = express_1.default();
        this.routes = new routes_1.default(data);
        this.bindMiddleware();
        this.bindDashboardRoute();
        this.bindAppRoute();
    }
    bindMiddleware() {
        this.app.set("views", path_1.default.resolve(__dirname, "./views"));
        this.app.set("view engine", "ejs");
        this.app.use(cors_1.default());
        this.app.use((req, res, next) => {
            log_1.default.log(`${moment_1.default.utc().format()} : ${log_1.default.blue(`[${req.method}] ${req.url}`)}`);
            next();
        });
    }
    bindDashboardRoute() {
        this.app.get("/", (req, res) => {
            return res.render("index", {
                port: this.port,
                host: this.host,
                routes: this.routes.routes,
            });
        });
    }
    bindAppRoute() {
        this.app.all("*", (req, res) => {
            const [path, methodList] = Object.entries(this.routes.routes).find(([path]) => this.doesMatchPath(path, req.url)) || [null, null];
            if (path === null) {
                return res.status(404).json(null);
            }
            if (!methodList[req.method.toLowerCase()]) {
                log_1.default.log(log_1.default.red(`route ${path} does not have method ${req.method}`));
                return res.status(404).json(null);
            }
            return res.jsonp(methodList[req.method.toLowerCase()]);
        });
        return this;
    }
    start() {
        this.server = this.app.listen(this.port, this.host, () => {
            log_1.default.log(`App Listening on ${log_1.default.green(`http://${this.host}:${this.port}`)}`);
        });
        return this;
    }
    stop() {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.close();
    }
    reload() {
        log_1.default.log(log_1.default.blue(".reload() has been deprecated."));
    }
    doesMatchPath(rawPath, serverPath) {
        return !!path_to_regexp_1.match(rawPath, { decode: decodeURIComponent })(serverPath);
    }
    static create(port, host, data) {
        return new Server(port, host, data);
    }
}
exports.default = Server;
//# sourceMappingURL=server.js.map