"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const log_1 = __importDefault(require("./log"));
const routes_1 = __importDefault(require("./routes"));
const path_to_regexp_1 = require("path-to-regexp");
class Server {
    constructor(port, host, data) {
        this.port = port;
        this.host = host;
        this.app = express_1.default();
        this.routes = new routes_1.default(data);
    }
    bindAppRoute() {
        this.app.all("*", (req, res) => {
            const [path, methodList] = Object.entries(this.routes.routes).find(([path]) => this.doesMatchPath(path, req.url)) || [null, null];
            if (path === null) {
                return res.status(404).json(null);
            }
            if (!methodList[req.method.toLowerCase()]) {
                log_1.default.log(log_1.default.red(`route ${path} does not have method ${req.method}`));
                return res.status(404).json(null);
            }
            return res.jsonp(methodList[req.method.toLowerCase()]);
        });
        return this;
    }
    start() {
        this.server = this.app.listen(this.port, this.host, () => {
            log_1.default.log(`App Listening on ${log_1.default.green(`http://${this.host}:${this.port}`)}`);
        });
        return this;
    }
    stop() {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.close();
    }
    reload() {
        log_1.default.log(log_1.default.blue(".reload() has been deprecated."));
    }
    doesMatchPath(rawPath, serverPath) {
        return !!path_to_regexp_1.match(rawPath, { decode: decodeURIComponent })(serverPath);
    }
    static create(port, host, data) {
        return new Server(port, host, data);
    }
}
exports.default = Server;
//# sourceMappingURL=server.js.map